@@ -1,3 +1,5 @@
 from .gfdl_plev19_vertical_coord import gfdl_plev19_vertical_coord
 from .gfdl_vertical_coord import gfdl_vertical_coord
+from .mom6_z_coord import mom6_z_coord
+from .mom6_rho2_coord import mom6_rho2_coord
 from .ncar_hybrid_coord import ncar_hybrid_coord
