@@ -4,11 +4,8 @@ ___all__ = [
     "gfdl_plev19_vertical_coord",
 ]
 
-import cftime
 import xarray as xr
 import numpy as np
-from mdtf_test_data.util.rectilinear import construct_rect_grid
-import mdtf_test_data.generators as generators
 
 
 def gfdl_plev19_vertical_coord():
